"""
Populate your database with god-like powers
"""

__author__ = "PeopleDoc"
__author_email__ = "yannick.peroux@people-doc.com"
__copyright__ = "Copyright 2017, PeopleDoc"
__license__ = "MIT"
__url__ = 'http://github.com/novafloss/populous'
__version__ = '0.2.2'
